from typing import Any, List


class ChannelInfo:
    def __init__(self, response: Any) -> None:
        self._response = response

    @property
    def id(self) -> str:
        return self._response["id"]

    @property
    def name(self) -> str:
        return self._response["name"]

    @property
    def english_name(self) -> str:
        return self._response["english_name"]

    @property
    def description(self) -> str:
        return self._response["description"]

    @property
    def photo(self) -> str:
        return self._response["photo"]

    @property
    def thumbnail(self) -> str:
        return self._response["thumbnail"]

    @property
    def banner(self) -> str:
        return self._response["banner"]

    @property
    def org(self) -> str:
        return self._response["org"]

    @property
    def suborg(self) -> str:
        return self._response["suborg"]

    """
    @property
    def lang(self) -> Optional[]
    """

    @property
    def published_at(self) -> str:
        return self._response["published_at"]

    @property
    def view_count(self) -> str:
        return self._response["view_count"]

    @property
    def video_count(self) -> str:
        return self._response["video_count"]

    @property
    def subscriber_count(self) -> str:
        return self._response["subscriber_count"]

    @property
    def comments_crawled_at(self) -> str:
        return self._response["comments_crawled_at"]

    @property
    def updated_at(self) -> str:
        return self._response["updated_at"]

    @property
    def yt_uploads_id(self) -> str:
        return self._response["yt_uploads_id"]

    @property
    def crawled_at(self) -> str:
        return self._response["crawled_at"]

    @property
    def type(self) -> str:
        return self._response["type"]

    @property
    def clip_count(self) -> int:
        return self._response["clip_count"]

    @property
    def twitter(self) -> str:
        return self._response["twitter"]

    @property
    def inactive(self) -> bool:
        return self._response["inactive"]

    @property
    def created_at(self) -> str:
        return self._response["created_at"]

    @property
    def top_topics(self) -> List[str]:
        return self._response["top_topics"]



from holodex.model.live import Live
from holodex.model.autocomplete import AutoComplete
from typing import Literal, Optional
from aiohttp.client import ClientSession

from holodex.http import HolodexHttpClient
from holodex.model.channel_info import ChannelInfo


class HolodexClient(HolodexHttpClient):
    def __init__(self, session: Optional[ClientSession] = None) -> None:
        super().__init__(session)

    async def channel_info(self, channel_id: str) -> ChannelInfo:
        return ChannelInfo(await self.channels(channel_id))

    async def autocomplete(self, keyword: str) -> AutoComplete:
        return AutoComplete(await self.search_autocomplete(keyword))

    async def live_streams(
        self,
        org: Optional[
            Literal["All Vtubers", "Hololive", "Nijisanji", "Independents"]
        ] = "All Vtubers",
    ) -> Live:
        return Live(await self.live(org))



from aiohttp import ClientSession
from typing import Any, Literal, Optional


class HolodexHttpClient:
    BASE_URL = "https://holodex.net/api/v2/"

    def __init__(self, session: Optional[ClientSession] = None) -> None:
        self.session = session

    async def close(self):
        if self.session:
            await self.session.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    async def request(
        self, method: Literal["GET", "POST"], endpoint: str, **kwargs: Any
    ) -> Any:
        if not self.session:
            self.session = ClientSession()

        async with self.session.request(
            method, self.BASE_URL + endpoint, **kwargs
        ) as r:
            return await r.json()

    async def channels(self, channel_id: str) -> Any:
        return await self.request("GET", f"/channels/{channel_id}")

    async def search_autocomplete(self, q: str) -> Any:
        return await self.request("GET", f"/search/autocomplete", params={"q": q})

    async def live(
        self,
        org: Literal["All Vtubers", "Hololive", "Nijisanji", "Independents"],
    ) -> Any:
        return await self.request("GET", f"/live", params={"org": org})



from typing import Any, Dict


class Channel:
    def __init__(self, data: Dict[str, Any]) -> None:
        self._data = data

    @property
    def id(self) -> str:
        return self._data["id"]

    @property
    def name(self) -> str:
        return self._data["name"]

    @property
    def org(self) -> str:
        return self._data["org"]

    @property
    def type(self) -> str:
        return self._data["type"]

    @property
    def photo(self) -> str:
        return self._data["photo"]

    @property
    def english_name(self) -> str:
        return self._data["english_name"]


class LiveInfo:
    def __init__(self, response: Any) -> None:
        self._response = response

    @property
    def id(self) -> str:
        return self._response["id"]

    @property
    def title(self) -> str:
        return self._response["title"]

    @property
    def type(self) -> str:
        return self._response["type"]

    @property
    def topic_id(self) -> str:
        return self._response["topic_id"]

    @property
    def published_at(self) -> str:
        return self._response["published_at"]

    @property
    def available_at(self) -> str:
        return self._response["available_at"]

    @property
    def duration(self) -> int:
        return self._response["duration"]

    @property
    def status(self) -> str:
        return self._response["status"]

    @property
    def live_tl_count(self) -> Dict[str, int]:
        return self._response["live_tl_count"]

    @property
    def start_scheduled(self) -> str:
        return self._response["start_scheduled"]

    @property
    def start_actual(self) -> str:
        return self._response["start_actual"]

    @property
    def live_viewers(self) -> int:
        return self._response["live_viewers"]

    @property
    def channel(self) -> Channel:
        return Channel(self._response["channel"])


class Live:
    def __init__(self, response: Any) -> None:
        self._response = response

    @property
    def contents(self) -> list[LiveInfo]:
        return [LiveInfo(content) for content in self._response]
